@@ -68,9 +68,12 @@ error = compare x y == GT ==> x > y
 error = head (sort x) ==> minimum x
 error = last (sort x) ==> maximum x
 error = head (sortBy f x) ==> minimumBy f x
+    where _ = isCompare f
 error = last (sortBy f x) ==> maximumBy f x
+    where _ = isCompare f
 error "Avoid reverse" = reverse (sort x) ==> sortBy (flip compare) x
 error "Avoid reverse" = reverse (sortBy f x) ==> sortBy (flip f) x
+    where _ = isCompare f
 warn  = flip (g `on` h) ==> flip g `on` h
 warn  = (f `on` g) `on` h ==> f `on` (g . h)
 
@@ -88,11 +91,16 @@ warn = showIntAtBase 8 intToDigit ==> showOct
 error = concat (map f x) ==> concatMap f x
 warn = concat [a, b] ==> a ++ b
 warn "Use map once" = map f (map g x) ==> map (f . g) x
+warn "Fuse concatMap/map" = concatMap f (map g x) ==> concatMap (f . g) x
 warn  = x !! 0 ==> head x
 error = take n (repeat x) ==> replicate n x
+    where _ = noQuickCheck -- takes too long
 error = map f (replicate n x) ==> replicate n (f x)
+    where _ = noQuickCheck -- takes too long
 error = map f (repeat x) ==> repeat (f x)
+    where _ = noQuickCheck -- takes forever
 error = cycle [x] ==> repeat x
+    where _ = noQuickCheck -- takes forever
 error = head (reverse x) ==> last x
 error = head (drop n x) ==> x !! n where _ = isNat n
 error = reverse (tail (reverse x)) ==> init x where note = IncreasesLaziness
@@ -111,7 +119,6 @@ error = snd (span p x) ==> dropWhile p x
 error = fst (break p x) ==> takeWhile (not . p) x
 error = snd (break p x) ==> dropWhile (not . p) x
 error = concatMap (++ "\n") ==> unlines
-error = intercalate "\n" x ++ "\n" ==> unlines x
 error = map id ==> id
 error = or (map p x) ==> any p x
 error = and (map p x) ==> all p x
@@ -156,6 +163,10 @@ error "Take on a non-positive" = take i x ==> [] where _ = isNegZero i
 error "Drop on a non-positive" = drop i x ==> x where _ = isNegZero i
 error = last (scanl f z x) ==> foldl f z x
 error = head (scanr f z x) ==> foldr f z x
+error = iterate id ==> repeat
+    where _ = noQuickCheck -- takes forever
+error = zipWith f (repeat x) ==> map (f x)
+error = zipWith f y (repeat z) ==> map (\x -> f x z) y
 
 -- BY
 
@@ -172,9 +183,10 @@ error = unionBy (==) ==> union
 -- FOLDS
 
 error = foldr  (>>) (return ()) ==> sequence_
+    where _ = noQuickCheck
 error = foldr  (&&) True ==> and
 error = foldl  (&&) True ==> and where note = IncreasesLaziness
-error = foldr1 (&&)  ==> and where note = RemovesError "on []"
+error = foldr1 (&&)  ==> and where note = RemovesError "on []"; _ = noQuickCheck
 error = foldl1 (&&)  ==> and where note = RemovesError "on []"
 error = foldr  (||) False ==> or
 error = foldl  (||) False ==> or where note = IncreasesLaziness
@@ -193,6 +205,7 @@ error = foldr1 max   ==> maximum
 error = foldl1 min   ==> minimum
 error = foldr1 min   ==> minimum
 error = foldr mplus mzero ==> msum
+    where _ = noQuickCheck
 
 -- FUNCTION
 
@@ -204,7 +217,7 @@ warn "Use curry" = (\x y -> f (x,y)) ==> curry f
 warn "Use uncurry" = (\(x,y) -> f x y) ==> uncurry f where note = IncreasesLaziness
 error "Redundant $" = (($) . f) ==> f
 error "Redundant $" = (f $) ==> f
-warn  = (\x -> y) ==> const y where _ = isAtom y && notIn x y
+warn  = (\x -> y) ==> const y where _ = isAtom y && not (isWildcard y)
 error "Redundant flip" = flip f x y ==> f y x where _ = isApp original
 warn  = (\a b -> g (f a) (f b)) ==> g `Data.Function.on` f
 error "Evaluate" = id x ==> x
@@ -217,11 +230,8 @@ error = a >= 'a' && a <= 'z' ==> isAsciiLower a
 error = a >= 'A' && a <= 'Z' ==> isAsciiUpper a
 error = a >= '0' && a <= '9' ==> isDigit a
 error = a >= '0' && a <= '7' ==> isOctDigit a
-error = not (isControl a) ==> isPrint a
 error = isLower a || isUpper a ==> isAlpha a
 error = isUpper a || isLower a ==> isAlpha a
-error = isAlpha a || isDigit a ==> isAlphaNum a
-error = isDigit a || isAlpha a ==> isAlphaNum a
 
 -- BOOL
 
@@ -253,7 +263,8 @@ warn = and [x, y, z] ==> x && y && z
 error "Redundant if" = (if x then False else y) ==> not x && y where _ = notEq y True
 error "Redundant if" = (if x then y else True) ==> not x || y where _ = notEq y False
 error "Redundant not" = not (not x) ==> x
-error "Too strict if" = (if c then f x else f y) ==> f (if c then x else y) where note = IncreasesLaziness
+-- error "Too strict if" = (if c then f x else f y) ==> f (if c then x else y) where note = IncreasesLaziness
+-- also breaks types, see #87
 
 -- ARROW
 
@@ -269,70 +280,73 @@ warn "Redundant pair" = (fst x, snd x) ==>  x where note = DecreasesLaziness
 
 -- FUNCTOR
 
-error "Functor law" = fmap f (fmap g x) ==> fmap (f . g) x
-error "Functor law" = fmap id ==> id
+error "Functor law" = fmap f (fmap g x) ==> fmap (f . g) x where _ = noQuickCheck
+error "Functor law" = fmap id ==> id where _ = noQuickCheck
 warn = fmap f $ x ==> f Control.Applicative.<$> x
-    where _ = isApp x || isAtom x
+    where _ = (isApp x || isAtom x) && noQuickCheck
 
 -- MONAD
 
-error "Monad law, left identity" = return a >>= f ==> f a
-error "Monad law, right identity" = m >>= return ==> m
-warn  = m >>= return . f ==> Control.Monad.liftM f m -- cannot be fmap, because is in Functor not Monad
-error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y where _ = not (isAtom y)
-error = (if x then y else return ()) ==> Control.Monad.when x y where _ = isAtom y
-error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y where _ = not (isAtom y)
-error = (if x then return () else y) ==> Control.Monad.unless x y where _ = isAtom y
-error = sequence (map f x) ==> mapM f x
-error = sequence_ (map f x) ==> mapM_ f x
-warn  = flip mapM ==> Control.Monad.forM
-warn  = flip mapM_ ==> Control.Monad.forM_
-warn  = flip forM ==> mapM
-warn  = flip forM_ ==> mapM_
-error = when (not x) ==> unless x
-error = x >>= id ==> Control.Monad.join x
-error = liftM f (liftM g x) ==> liftM (f . g) x
-error = fmap f (fmap g x) ==> fmap (f . g) x
+error "Monad law, left identity" = return a >>= f ==> f a where _ = noQuickCheck
+error "Monad law, right identity" = m >>= return ==> m where _ = noQuickCheck
+warn  = m >>= return . f ==> Control.Monad.liftM f m where _ = noQuickCheck -- cannot be fmap, because is in Functor not Monad
+error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y where _ = not (isAtom y) && noQuickCheck
+error = (if x then y else return ()) ==> Control.Monad.when x y where _ = isAtom y && noQuickCheck
+error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y where _ = not (isAtom y) && noQuickCheck
+error = (if x then return () else y) ==> Control.Monad.unless x y where _ = isAtom y && noQuickCheck
+error = sequence (map f x) ==> mapM f x where _ = noQuickCheck
+error = sequence_ (map f x) ==> mapM_ f x where _ = noQuickCheck
+warn  = flip mapM ==> Control.Monad.forM where _ = noQuickCheck
+warn  = flip mapM_ ==> Control.Monad.forM_ where _ = noQuickCheck
+warn  = flip forM ==> mapM where _ = noQuickCheck
+warn  = flip forM_ ==> mapM_ where _ = noQuickCheck
+error = when (not x) ==> unless x where _ = noQuickCheck
+error = x >>= id ==> Control.Monad.join x where _ = noQuickCheck
+error = liftM f (liftM g x) ==> liftM (f . g) x where _ = noQuickCheck
+error = fmap f (fmap g x) ==> fmap (f . g) x where _ = noQuickCheck
 warn  = a >> return () ==> Control.Monad.void a
-    where _ = isAtom a || isApp a
-error = fmap (const ()) ==> Control.Monad.void
-error = flip (>=>) ==> (<=<)
-error = flip (<=<) ==> (>=>)
-warn  = (\x -> f x >>= g) ==> f Control.Monad.>=> g
-warn  = (\x -> f =<< g x) ==> f Control.Monad.<=< g
-error = a >> forever a ==> forever a
-warn  = liftM2 id ==> ap
-error = mapM (uncurry f) (zip l m) ==> zipWithM f l m
+    where _ = (isAtom a || isApp a) && noQuickCheck
+error = fmap (const ()) ==> Control.Monad.void where _ = noQuickCheck
+error = flip (>=>) ==> (<=<) where _ = noQuickCheck
+error = flip (<=<) ==> (>=>) where _ = noQuickCheck
+error = flip (>>=) ==> (=<<) where _ = noQuickCheck
+error = flip (=<<) ==> (>>=) where _ = noQuickCheck
+warn  = (\x -> f x >>= g) ==> f Control.Monad.>=> g where _ = noQuickCheck
+warn  = (\x -> f =<< g x) ==> f Control.Monad.<=< g where _ = noQuickCheck
+error = a >> forever a ==> forever a where _ = noQuickCheck
+warn  = liftM2 id ==> ap where _ = noQuickCheck
+error = mapM (uncurry f) (zip l m) ==> zipWithM f l m where _ = noQuickCheck
 
 -- STATE MONAD
 
-error = fst (runState x y) ==> evalState x y
-error = snd (runState x y) ==> execState x y
+error = fst (runState x y) ==> evalState x y where _ = noQuickCheck
+error = snd (runState x y) ==> execState x y where _ = noQuickCheck
 
 -- MONAD LIST
 
-error = liftM unzip (mapM f x) ==> Control.Monad.mapAndUnzipM f x
-error = sequence (zipWith f x y) ==> Control.Monad.zipWithM f x y
-error = sequence_ (zipWith f x y) ==> Control.Monad.zipWithM_ f x y
-error = sequence (replicate n x) ==> Control.Monad.replicateM n x
-error = sequence_ (replicate n x) ==> Control.Monad.replicateM_ n x
-error = mapM f (replicate n x) ==> Control.Monad.replicateM n (f x)
-error = mapM_ f (replicate n x) ==> Control.Monad.replicateM_ n (f x)
-error = mapM f (map g x) ==> mapM (f . g) x
-error = mapM_ f (map g x) ==> mapM_ (f . g) x
-error = mapM id ==> sequence
-error = mapM_ id ==> sequence_
+error = liftM unzip (mapM f x) ==> Control.Monad.mapAndUnzipM f x where _ = noQuickCheck
+error = sequence (zipWith f x y) ==> Control.Monad.zipWithM f x y where _ = noQuickCheck
+error = sequence_ (zipWith f x y) ==> Control.Monad.zipWithM_ f x y where _ = noQuickCheck
+error = sequence (replicate n x) ==> Control.Monad.replicateM n x where _ = noQuickCheck
+error = sequence_ (replicate n x) ==> Control.Monad.replicateM_ n x where _ = noQuickCheck
+error = mapM f (replicate n x) ==> Control.Monad.replicateM n (f x) where _ = noQuickCheck
+error = mapM_ f (replicate n x) ==> Control.Monad.replicateM_ n (f x) where _ = noQuickCheck
+error = mapM f (map g x) ==> mapM (f . g) x where _ = noQuickCheck
+error = mapM_ f (map g x) ==> mapM_ (f . g) x where _ = noQuickCheck
+error = mapM id ==> sequence where _ = noQuickCheck
+error = mapM_ id ==> sequence_ where _ = noQuickCheck
 
 -- APPLICATIVE / TRAVERSABLE
 
-error = flip traverse ==> for
-error = flip for ==> traverse
-error = flip traverse_ ==> for_
-error = flip for_ ==> traverse_
-error = foldr (*>) (pure ()) ==> sequenceA_
-error = foldr (<|>) empty ==> asum
-error = liftA2 (flip ($)) ==> (<**>)
-error = Just <$> a <|> pure Nothing ==> optional a
+error = flip traverse ==> for where _ = noQuickCheck
+error = flip for ==> traverse where _ = noQuickCheck
+error = flip traverse_ ==> for_ where _ = noQuickCheck
+error = flip for_ ==> traverse_ where _ = noQuickCheck
+error = foldr (*>) (pure ()) ==> sequenceA_ where _ = noQuickCheck
+error = foldr (<|>) empty ==> asum where _ = noQuickCheck
+error = liftA2 (flip ($)) ==> (<**>) where _ = noQuickCheck
+error = Just <$> a <|> pure Nothing ==> optional a where _ = noQuickCheck
+
 
 -- LIST COMP
 
@@ -374,6 +388,7 @@ error = (if isJust x then fromJust x else y) ==> fromMaybe y x
 error = isJust x && (fromJust x == y) ==> x == Just y
 error = mapMaybe f (map g x) ==> mapMaybe (f . g) x
 error = fromMaybe a (fmap f x) ==> maybe a f x
+error = mapMaybe id ==> catMaybes
 warn = [x | Just x <- a] ==> Data.Maybe.catMaybes a
 warn = (case m of Nothing -> Nothing; Just x -> x) ==> Control.Monad.join m
 warn = maybe Nothing id ==> join
@@ -383,6 +398,7 @@ warn "Too strict maybe" = maybe (f x) (f . g) ==> f . maybe x g where note = Inc
 
 error = [a | Left a <- a] ==> lefts a
 error = [a | Right a <- a] ==> rights a
+error = either Left (Right . f) ==> fmap f
 
 -- INFIX
 
@@ -403,7 +419,6 @@ warn  = 0 - x ==> negate x
 error "Redundant negate" = negate (negate x) ==> x
 warn  = log y / log x ==> logBase x y
 warn  = sin x / cos x ==> tan x
-warn  = sinh x / cosh x ==> tanh x
 warn  = n `rem` 2 == 0 ==> even n
 warn  = n `rem` 2 /= 0 ==> odd n
 warn  = not (even x) ==> odd x
@@ -437,7 +452,9 @@ error = mkWeak a (a, b) c ==> mkWeakPair a b c
 -- FOLDABLE
 
 error "Use Foldable.forM_" = (case m of Nothing -> return (); Just x -> f x) ==> Data.Foldable.forM_ m f
+    where _ = noQuickCheck
 error "Use Foldable.forM_" = when (isJust m) (f (fromJust m)) ==> Data.Foldable.forM_ m f
+    where _ = noQuickCheck
 
 -- EVALUATE
 
@@ -574,6 +591,7 @@ getInt = do { x <- readIO "0"; return $! (x :: Int) }
 foo = evaluate [12] -- return [12]
 test = \ a -> f a >>= \ b -> return (a, b)
 fooer input = catMaybes . map Just $ input -- mapMaybe Just
+yes = mapMaybe id -- catMaybes
 main = print $ map (\_->5) [2,3,5] -- const 5
 main = head $ drop n x
 main = head $ drop (-3) x -- x
@@ -605,6 +623,18 @@ foo = return $! 1
 foo = return $! "test"
 bar = [x| (x,_) <- pts]
 return' x = x `seq` return x
+foo = last (sortBy (compare `on` fst) xs) -- maximumBy (compare `on` fst) xs
+g = \ f -> parseFile f >>= (\ cu -> return (f, cu))
+foo = bar $ \(x,y) -> x x y
+foo = (\x -> f x >>= g) -- f Control.Monad.>=> g
+foo = (\f -> h f >>= g) -- h Control.Monad.>=> g
+foo = (\f -> h f >>= f)
+foo = bar $ \x -> [x,y]
+foo = bar $ \x -> [z,y] -- const [z,y]
+f condition tChar tBool = if condition then _monoField tChar else _monoField tBool
+foo = maybe Bar{..} id -- Data.Maybe.fromMaybe Bar{..}
+foo = (\a -> Foo {..}) 1
+foo = zipWith SymInfo [0 ..] (repeat ty) -- map (\ x -> SymInfo x ty) [0 ..]
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
