@@ -1,18 +1,21 @@
 {-# LANGUAGE PatternGuards, RecordWildCards, DeriveDataTypeable #-}
-{-# OPTIONS_GHC -fno-warn-missing-fields #-}
+{-# OPTIONS_GHC -fno-warn-missing-fields -fno-cse #-}
 
-module CmdLine(Cmd(..), cmdCpp, CppFlags(..), getCmd, cmdExtensions, cmdHintFiles, exitWithHelp, resolveFile) where
+module CmdLine(Cmd(..), cmdCpp, CppFlags(..), getCmd, cmdExtensions, cmdHintFiles, cmdUseColour, exitWithHelp, resolveFile) where
 
 import Data.Char
 import Data.List
+import System.Console.ANSI(hSupportsANSI)
 import System.Console.CmdArgs.Implicit
 import System.Console.CmdArgs.Explicit(helpText, HelpFormat(..))
-import System.Directory
+import System.Directory.Extra
 import System.Exit
 import System.FilePath
+import System.IO
 import Language.Preprocessor.Cpphs
 import Language.Haskell.Exts.Extension
 import System.Environment
+import System.Info.Extra
 
 import Util
 import Paths_hlint
@@ -52,13 +55,25 @@ data CppFlags
     | Cpphs CpphsOptions -- ^ The @cpphs@ library is used.
 
 
+-- | When to colour terminal output.
+data ColorMode
+    = Never  -- ^ Terminal output will never be coloured.
+    | Always -- ^ Terminal output will always be coloured.
+    | Auto   -- ^ Terminal output will be coloured if $TERM and stdout appear to support it.
+      deriving (Show, Typeable, Data)
+
+
+instance Default ColorMode where
+  def = if isWindows then Never else Auto
+
+
 data Cmd
     = CmdMain
         {cmdFiles :: [FilePath]    -- ^ which files to run it on, nothing = none given
         ,cmdReports :: [FilePath]        -- ^ where to generate reports
         ,cmdGivenHints :: [FilePath]     -- ^ which settignsfiles were explicitly given
         ,cmdWithHints :: [String]        -- ^ hints that are given on the command line
-        ,cmdColor :: Bool                -- ^ color the result
+        ,cmdColor :: ColorMode           -- ^ color the result
         ,cmdIgnore :: [String]           -- ^ the hints to ignore
         ,cmdShowAll :: Bool              -- ^ display all skipped items
         ,cmdExtension :: [String]        -- ^ extensions
@@ -74,6 +89,8 @@ data Cmd
         ,cmdCppFile :: [FilePath]
         ,cmdCppSimple :: Bool
         ,cmdCppAnsi :: Bool
+        ,cmdJson :: Bool                -- ^ display hint data as JSON
+        ,cmdNoSummary :: Bool           -- ^ do not show the summary info
         }
     | CmdGrep
         {cmdFiles :: [FilePath]    -- ^ which files to run it on, nothing = none given
@@ -95,9 +112,13 @@ data Cmd
         ,cmdDataDir :: FilePath          -- ^ the data directory
         ,cmdReports :: [FilePath]        -- ^ where to generate reports
         ,cmdWithHints :: [String]        -- ^ hints that are given on the command line
+        ,cmdTempDir :: FilePath          -- ^ temporary directory to put the files in
+        ,cmdQuickCheck :: Bool
+        ,cmdTypeCheck :: Bool
         }
     | CmdHSE
         {cmdFiles :: [FilePath]
+        ,cmdLanguage :: [String]      -- ^ the extensions (may be prefixed by "No")
         }
     deriving (Data,Typeable,Show)
 
@@ -107,7 +128,7 @@ mode = cmdArgsMode $ modes
         ,cmdReports = nam "report" &= opt "report.html" &= typFile &= help "Generate a report in HTML"
         ,cmdGivenHints = nam "hint" &= typFile &= help "Hint/ignore file to use"
         ,cmdWithHints = nam "with" &= typ "HINT" &= help "Extra hints to use"
-        ,cmdColor = nam "colour" &= name "color" &= help "Color output (requires ANSI terminal)"
+        ,cmdColor = nam "colour" &= name "color" &= opt Always &= typ "always/never/auto" &= help "Color output (requires ANSI terminal; auto means on when $TERM is supported; by itself, selects always)"
         ,cmdIgnore = nam "ignore" &= typ "HINT" &= help "Ignore a particular hint"
         ,cmdShowAll = nam "show" &= help "Show all ignored ideas"
         ,cmdExtension = nam "extension" &= typ "EXT" &= help "File extensions to search (default hs/lhs)"
@@ -120,9 +141,11 @@ mode = cmdArgsMode $ modes
         ,cmdPath = nam "path" &= help "Directory in which to search for files"
         ,cmdCppDefine = nam_ "cpp-define" &= typ "NAME[=VALUE]" &= help "CPP #define"
         ,cmdCppInclude = nam_ "cpp-include" &= typDir &= help "CPP include path"
-        ,cmdCppFile = nam_ "cpp-file" &= typDir &= help "CPP pre-include file"
+        ,cmdCppFile = nam_ "cpp-file" &= typFile &= help "CPP pre-include file"
         ,cmdCppSimple = nam_ "cpp-simple" &= help "Use a simple CPP (strip # lines)"
         ,cmdCppAnsi = nam_ "cpp-ansi" &= help "Use CPP in ANSI compatibility mode"
+        ,cmdJson = nam_ "json" &= help "Display hint data as JSON"
+        ,cmdNoSummary = nam_ "no-summary" &= help "Do not show summary information"
         } &= auto &= explicit &= name "lint"
     ,CmdGrep
         {cmdFiles = def &= args &= typ "FILE/DIR"
@@ -130,6 +153,9 @@ mode = cmdArgsMode $ modes
         } &= explicit &= name "grep"
     ,CmdTest
         {cmdProof = nam_ "proof" &= typFile &= help "Isabelle/HOLCF theory file"
+        ,cmdTypeCheck = nam_ "typecheck" &= help "Use GHC to type check the hints"
+        ,cmdQuickCheck = nam_ "quickcheck" &= help "Use QuickCheck to check the hints"
+        ,cmdTempDir = nam_ "tempdir" &= help "Where to put temporary files (not cleaned up)"
         } &= explicit &= name "test"
         &= details ["HLint gives hints on how to improve Haskell code."
                  ,""
@@ -138,9 +164,9 @@ mode = cmdArgsMode $ modes
     ,CmdHSE
         {} &= explicit &= name "hse"
     ] &= program "hlint" &= verbosity
-    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2014")
+    &=  summary ("HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2015")
     where
-        nam xs@(x:_) = nam_ xs &= name [x]
+        nam xs = nam_ xs &= name [head xs]
         nam_ xs = def &= explicit &= name xs
 
 cmdHintFiles :: Cmd -> IO [FilePath]
@@ -162,6 +188,14 @@ cmdCpp cmd
     | otherwise = NoCpp
 
 
+-- | Determines whether to use colour or not.
+cmdUseColour :: Cmd -> IO Bool
+cmdUseColour cmd = case cmdColor cmd of
+  Always -> return True
+  Never  -> return False
+  Auto   -> hSupportsANSI stdout
+
+
 "." <\> x = x
 x <\> y = x </> y
 
@@ -176,7 +210,8 @@ getFile [] exts file = error $ "Couldn't find file: " ++ file
 getFile (p:ath) exts file = do
     isDir <- doesDirectoryExist $ p <\> file
     if isDir then do
-        xs <- getDirectoryContentsRecursive $ p <\> file
+        let avoid x = let y = takeFileName x in "_" `isPrefixOf` y || ("." `isPrefixOf` y && not (all (== '.') y))
+        xs <- listFilesInside (return . not . avoid) $ p <\> file
         return [x | x <- xs, drop 1 (takeExtension x) `elem` exts]
      else do
         isFil <- doesFileExist $ p <\> file
