@@ -1,4 +1,3 @@
-{-# LANGUAGE RecordWildCards #-}
 
 module HSE.All(
     module X,
@@ -17,22 +16,23 @@ import Util
 import CmdLine
 import Control.Exception
 import Data.Char
-import Data.List
+import Data.List.Extra
 import Data.Maybe
 import Language.Preprocessor.Cpphs
 import qualified Data.Map as Map
+import System.IO
 
 
 -- | Created with 'defaultParseFlags', used by 'parseModuleEx'.
 data ParseFlags = ParseFlags
-    {encoding :: Encoding -- ^ How the file is read in (defaults to 'defaultEncoding').
+    {encoding :: TextEncoding -- ^ How the file is read in (defaults to 'utf8').
     ,cppFlags :: CppFlags -- ^ How the file is preprocessed (defaults to 'NoCpp').
     ,hseFlags :: ParseMode -- ^ How the file is parsed (defaults to all fixities in the @base@ package and most non-conflicting extensions).
     }
 
--- | Default values for 'ParseFlags'.
+-- | Default value for 'ParseFlags'.
 defaultParseFlags :: ParseFlags
-defaultParseFlags = ParseFlags defaultEncoding NoCpp defaultParseMode{fixities=Just baseFixities, ignoreLinePragmas=False, extensions=defaultExtensions}
+defaultParseFlags = ParseFlags utf8 NoCpp defaultParseMode{fixities=Just baseFixities, ignoreLinePragmas=False, extensions=defaultExtensions}
 
 parseFlagsNoLocations :: ParseFlags -> ParseFlags
 parseFlagsNoLocations x = x{cppFlags = case cppFlags x of Cpphs y -> Cpphs $ f y; y -> y}
@@ -48,7 +48,7 @@ parseFlagsSetExtensions es x = x{hseFlags=(hseFlags x){extensions = es}}
 
 runCpp :: CppFlags -> FilePath -> String -> IO String
 runCpp NoCpp _ x = return x
-runCpp CppSimple _ x = return $ unlines [if "#" `isPrefixOf` ltrim x then "" else x | x <- lines x]
+runCpp CppSimple _ x = return $ unlines [if "#" `isPrefixOf` trimStart x then "" else x | x <- lines x]
 runCpp (Cpphs o) file x = runCpphs o file x
 
 
@@ -64,12 +64,12 @@ data ParseError = ParseError
 
 -- | Parse a Haskell module. Applies the C pre processor, and uses best-guess fixity resolution if there are ambiguities.
 --   The filename @-@ is treated as @stdin@. Requires some flags (often 'defaultParseFlags'), the filename, and optionally the contents of that file.
-parseModuleEx :: ParseFlags -> FilePath -> Maybe String -> IO (Either ParseError (Module SrcSpanInfo))
+parseModuleEx :: ParseFlags -> FilePath -> Maybe String -> IO (Either ParseError (Module SrcSpanInfo, [Comment]))
 parseModuleEx flags file str = do
         str <- maybe (readFileEncoding (encoding flags) file) return str
         ppstr <- runCpp (cppFlags flags) file str
-        case parseFileContentsWithMode (mode flags) ppstr of
-            ParseOk x -> return $ Right $ applyFixity fixity x
+        case parseFileContentsWithComments (mode flags) ppstr of
+            ParseOk (x, cs) -> return $ Right (applyFixity fixity x, cs)
             ParseFailed sl msg -> do
                 -- figure out the best line number to grab context from, by reparsing
                 flags <- return $ parseFlagsNoLocations flags
@@ -90,7 +90,7 @@ parseModuleEx flags file str = do
 -- | Given a line number, and some source code, put bird ticks around the appropriate bit.
 context :: Int -> String -> String
 context lineNo src =
-    unlines $ trimBy (all isSpace) $
+    unlines $ dropWhileEnd (all isSpace) $ dropWhile (all isSpace) $
     zipWith (++) ticks $ take 5 $ drop (lineNo - 3) $ lines src ++ ["","","","",""]
     where ticks = ["  ","  ","> ","  ","  "]
 
