@@ -3,7 +3,7 @@
 module Hint.Util where
 
 import HSE.All
-import Util
+import Data.List.Extra
 
 
 -- | Generate a lambda, but prettier (if possible).
@@ -22,7 +22,7 @@ niceLambda xs (Lambda _ [] x) = niceLambda xs x
 niceLambda [] x = x
 
 -- \xs -> e xs ==> e
-niceLambda xs (fromApps -> e) | map view xs2 == map Var_ xs, vars e2 `disjoint` xs, notNull e2 = apps e2
+niceLambda xs (fromApps -> e) | map view xs2 == map Var_ xs, vars e2 `disjoint` xs, not $ null e2 = apps e2
     where (e2,xs2) = splitAt (length e - length xs) e
 
 -- \x y -> x + y ==> (+)
@@ -42,8 +42,8 @@ niceLambda [x,y] (view -> App2 op (view -> Var_ y1) (view -> Var_ x1))
 niceLambda [x] y | Just z <- factor y, x `notElem` vars z = z
     where
         -- factor the expression with respect to x
-        factor y@App{} | (ini,lst) <- unsnoc $ fromApps y, view lst == Var_ x = Just $ apps ini
-        factor y@App{} | (ini,lst) <- unsnoc $ fromApps y, Just z <- factor lst = Just $ niceDotApp (apps ini) z
+        factor y@App{} | Just (ini,lst) <- unsnoc $ fromApps y, view lst == Var_ x = Just $ apps ini
+        factor y@App{} | Just (ini,lst) <- unsnoc $ fromApps y, Just z <- factor lst = Just $ niceDotApp (apps ini) z
         factor (InfixApp _ y op (factor -> Just z)) | isDol op = Just $ niceDotApp y z
         factor (Paren _ y@App{}) = factor y
         factor _ = Nothing
@@ -67,7 +67,7 @@ niceDotApp a b | a ~= "$" = b
 --   Mainly so that later stages can match on fewer alternatives.
 simplifyExp :: Exp_ -> Exp_
 simplifyExp (InfixApp _ x dol y) | isDol dol = App an x (paren y)
-simplifyExp (Let _ (BDecls _ [PatBind _ (view -> PVar_ x) Nothing (UnGuardedRhs _ y) Nothing]) z)
+simplifyExp (Let _ (BDecls _ [PatBind _ (view -> PVar_ x) (UnGuardedRhs _ y) Nothing]) z)
     | x `notElem` vars y && length [() | UnQual _ a <- universeS z, prettyPrint a == x] <= 1 = transform f z
     where f (view -> Var_ x') | x == x' = paren y
           f x = x
